import math
import random
import numpy as np
seedDivider = 4

#=============================================================================== 79
# Mathblob
def generateL79(seed, maxPoints):
    A=sin_osc(seed/67611,.1,1) #amplitude
    wavelength=sin_osc(seed/18761,.2,2)
    period=5#sin_osc(seed/314,1,15)
    t=seed/1721.76 #time
    grid_size=.5 #Size of the grid (-grid_size to +grid_size)
    yresolution=12
    xresolution=abs(int((sin_osc(seed/13451+sin_osc(seed/24319,0,10),0,1)*1000)/yresolution))+2

     # Calculate wave parameters
    k = 2 * math.pi / wavelength  # Wave number
    omega = 2 * math.pi / period  # Angular frequency

    # Generate grid points
    x = np.linspace(-grid_size, grid_size, xresolution + 1)
    y = np.linspace(-grid_size, grid_size, yresolution + 1)
    X, Y = np.meshgrid(x, y)

    # Calculate radial distances and wave heights
    R = np.sqrt(X**2 + Y**2)
    Z = A * np.sin(k * R - omega * t)

    # Convert to list of points in zigzag order
    wpoints = []
    for i in range(yresolution + 1):
        if i % 2 == 0:  # Even rows - left to right
            for j in range(xresolution + 1):
                wpoints.append((X[i,j], Y[i,j], Z[i,j]))
        else:  # Odd rows - right to left
            for j in range(xresolution, -1, -1):
                wpoints.append((X[i,j], Y[i,j], Z[i,j]))
    while(len(wpoints)<maxPoints): #append points for safety if necessary
        wpoints.append((0,0,0))
    points = []
    st=seed/1234.156
    for i in range(maxPoints):
        ii=int(i/(1000/(xresolution*yresolution)-.011))
        xa,ya,za = wpoints[ii]
        x,y=mathblobRotate(xa, ya, za, st)
        x=x+.5
        y=y+.5
        points.append((x,y,1))
    return points, "Mathblob"

def mathblobRotate(x, y, z, angle):
    rotation_matrix = np.array([
        [np.cos(angle), -np.sin(angle),0],
        [np.sin(angle), np.cos(angle),0],
        [0,0,1]
    ])
    point = np.array([x, y, z])
    rotated_point = np.dot(rotation_matrix, point)
    return rotated_point[0], rotated_point[2]

#=============================================================================== 78
# Barcode
def generateL78(seed, maxPoints):
    seed/=64
    points = []
    lCount=0
    rCount=0
    lrd = 0
    for i in range(maxPoints):
        y = i/maxPoints
        x = 0
        lr = (i+sin_osc(seed/3.519+i/15.156,-20,20)+sin_osc(seed/6.521+i/27.83,-20,20)+sin_osc(seed/34.57+i/23.67,-20,20)+sin_osc(seed/13.51+i/22.4,-20,20)+sin_osc(seed/133.51+i/21,-10,10)+sin_osc(seed/1.3981+i,-2,2)+sin_osc(seed/3.431+i,-2,2)+sin_osc(seed/2.784+i,-2,2))//100%2

        if lr==0:
            if lrd!=1:
                lrd=1
                lCount+=1
            x=(sin_osc(lCount+seed/899.2,0,.5)+sin_osc(lCount+seed/89.14,0,.06)+sin_osc(lCount+seed/8.123,0,.03))/10
        else:
            if lrd != -1:
                lrd=-1
                rCount+=1
            x=(sin_osc(rCount+seed/619.5,0,.1)+sin_osc(rCount+seed/61.31,0,.06)+sin_osc(rCount+seed/6.781,0,.03))/10+.8
        points.append((x,y,1))
    return points, "Barcode"
#=============================================================================== 77
# globlines
def generateL77(seed, maxPoints):
    seed/=12
    points = []

    xa1 = sin_osc(seed/401,0,1)
    ya1 = sin_osc(seed/472,0,1)
    xa2 = sin_osc(seed/463,0,1)
    ya2 = sin_osc(seed/458,0,1)

    xb1 = sin_osc(seed/407,0,1)
    yb1 = sin_osc(seed/416,0,1)
    xb2 = sin_osc(seed/435,0,1)
    yb2 = sin_osc(seed/444,0,1)


    for i in range(maxPoints):

        ii=i/maxPoints

        xa=lerp(ii,xa1,xa2)
        ya=lerp(ii,ya1,ya2)

        xb=lerp(ii,xb1,xb2)
        yb=lerp(ii,yb1,yb2)

        r=saw_osc(i/80+2)

        x=(lerp(r,xa,xb)*15)/5
        y=(lerp(r,ya,yb)*15)/5

        #x,y=project_to_sphere(x, y, 0, 0)
        w=seed/1345+i/1000
        x=sin_osc(x+w,0,1)
        y=sin_osc(y+w/1000,0,1)

        points.append((x,y,1))
    return points, "GlobLines"

#=============================================================================== 76
# connectedlines
def generateL76(seed, maxPoints):
    seed/=6
    points = []

    xa1 = sin_osc(seed/301,0,1)
    ya1 = sin_osc(seed/372,0,1)
    xa2 = sin_osc(seed/363,0,1)
    ya2 = sin_osc(seed/358,0,1)

    xb1 = sin_osc(seed/307,0,1)
    yb1 = sin_osc(seed/316,0,1)
    xb2 = sin_osc(seed/335,0,1)
    yb2 = sin_osc(seed/344,0,1)

    xc1 = sin_osc(seed/437,0,1)
    yc1 = sin_osc(seed/446,0,1)
    xc2 = sin_osc(seed/455,0,1)
    yc2 = sin_osc(seed/464,0,1)

    for i in range(maxPoints):
        x=0
        y=0
        ii=i/maxPoints
        d=(((i)//45)+((i+1)//45))%4
        if d==0:
            x=lerp(ii,xa1,xa2)
            y=lerp(ii,ya1,ya2)
        elif d==1:
            x=lerp(ii,xb1,xb2)
            y=lerp(ii,yb1,yb2)
        elif d==2:
            x=lerp(ii,xc1,xc2)
            y=lerp(ii,yc1,yc2)
        elif d==3:
            x=lerp(ii,xb1,xb2)
            y=lerp(ii,yb1,yb2)

        points.append((x,y,1))
    return points, "ConnectedLines"
#=============================================================================== 75
# manylines
def generateL75(seed, maxPoints):
    seed/=12
    points = []

    xa1 = sin_osc(seed/301,0,1)
    ya1 = sin_osc(seed/372,0,1)
    xa2 = sin_osc(seed/363,0,1)
    ya2 = sin_osc(seed/358,0,1)

    xb1 = sin_osc(seed/307,0,1)
    yb1 = sin_osc(seed/316,0,1)
    xb2 = sin_osc(seed/335,0,1)
    yb2 = sin_osc(seed/344,0,1)

    xc1 = sin_osc(seed/437,0,1)
    yc1 = sin_osc(seed/446,0,1)
    xc2 = sin_osc(seed/455,0,1)
    yc2 = sin_osc(seed/464,0,1)



    for i in range(maxPoints):
        x=0
        y=0
        ii=i/maxPoints
        d=((i+15)//30)%3
        if d==0:
            x=lerp(ii,xa1,xa2)
            y=lerp(ii,ya1,ya2)
        elif d==1:
            x=lerp(ii,xb1,xb2)
            y=lerp(ii,yb1,yb2)
        elif d==2:
            x=lerp(ii,xc1,xc2)
            y=lerp(ii,yc1,yc2)

        points.append((x,y,1))
    return points, "ManyLines"
#=============================================================================== 74
# lines
def generateL74(seed, maxPoints):
    seed/=12
    points = []

    xa1 = sin_osc(seed/301,0,1)
    ya1 = sin_osc(seed/372,0,1)
    xa2 = sin_osc(seed/363,0,1)
    ya2 = sin_osc(seed/358,0,1)

    xb1 = sin_osc(seed/307,0,1)
    yb1 = sin_osc(seed/316,0,1)
    xb2 = sin_osc(seed/335,0,1)
    yb2 = sin_osc(seed/344,0,1)


    for i in range(maxPoints):
        x=0
        y=0
        ii=i/maxPoints
        if ((i+15)//30)%2==0:
            x=lerp(ii,xa1,xa2)
            y=lerp(ii,ya1,ya2)
        else:
            x=lerp(ii,xb1,xb2)
            y=lerp(ii,yb1,yb2)

        points.append((x,y,1))
    return points, "Lines"
#=============================================================================== 73
# triaPlenty
def generateL73(seed, maxPoints):

    seed=seed/128
    points = []

    for i in range(maxPoints):
        poly = 3
        triAmount=soft_step(sin_osc(seed/8.4,2,12)+sin_osc(seed/18.14,2,12))/2
        repeats=soft_step(sin_osc(seed/61.1,1,6))/2
        triCount = int(i/(maxPoints-2)*poly*triAmount*repeats)/poly*math.pi*2
        rotation = soft_step(soft_step((seed+i/500)/3.96+sin_osc(seed/3.71,1,5)+sin_osc(seed/21.73,1,5)))
        triCount+=rotation*math.pi/1.5 #rotation
        triNumber = int((i-1)/maxPoints*triAmount)

        bigScale=sin_osc(seed/66.2,-1,1)*(i/maxPoints/2+.5)
        r=soft_step(soft_step(sin_osc(seed/8.23,0,4)+sin_osc(seed/62.1,0,4)))/4+1
        xShift =  math.cos(triNumber*r*math.pi/1.5)*bigScale + i/maxPoints*.2
        yShift =  math.sin(triNumber*r*math.pi/1.5)*bigScale + i/maxPoints*.2

        scale=sin_osc(triNumber+seed/11.61,.05,.5)

        x = math.sin(triCount)*scale+xShift
        y = math.cos(triCount)*scale+yShift

        x=(x)/2+.5
        y=(y)/2+.5

        points.append((x,y,1))
    return points, "TriPlenty"
#=============================================================================== 72
# hexaPlenty
def generateL72(seed, maxPoints):

    seed=seed/64
    points = []

    for i in range(maxPoints):
        poly = 6
        hexAmount=soft_step(sin_osc(seed/8.4,2,12)+sin_osc(seed/18.14,2,12))/2
        repeats=soft_step(sin_osc(seed/61.1,1,6))/2
        hexCount = int(i/(maxPoints-2)*poly*hexAmount*repeats)/poly*math.pi*2
        rotation = soft_step(soft_step((seed+i/500)/3.96+sin_osc(seed/3.71,1,5)+sin_osc(seed/21.73,1,5)))
        hexCount+=rotation*math.pi/6 #rotation
        hexNumber = int((i-1)/maxPoints*hexAmount)

        bigScale=sin_osc(seed/66.2,-1,1)
        r=soft_step(sin_osc(seed/8.23,0,4)+sin_osc(seed/62.1,0,4))/4
        xShift =  math.sin(hexNumber*r*math.pi/3)*bigScale + i/maxPoints*.2
        yShift =  math.cos(hexNumber*r*math.pi/3)*bigScale + i/maxPoints*.2

        scale=sin_osc(hexNumber+seed/11.61,.05,.5)

        x = math.sin(hexCount)*scale+xShift
        y = math.cos(hexCount)*scale+yShift

        x=(x)/2+.5
        y=(y)/2+.5

        points.append((x,y,1))
    return points, "HexaPlenty"

#=============================================================================== 71
# sunshineGears
def generateL71(seed, maxPoints):
    seed=seed/24
    points = []
    o1=soft_step(sin_osc(seed/445,1,6))
    o2=soft_step(sin_osc(seed/475,1,6))
    o9=soft_step(sin_osc(seed/573,1,6))
    o3=soft_step(sin_osc(seed/248.5,0,maxPoints/20))*20
    o4=soft_step(sin_osc(seed/143.5,0,maxPoints/20))*20
    o5=soft_step(sin_osc(seed/348.5,0,maxPoints/20))*20
    o6=soft_step(sin_osc(seed/443.5,0,maxPoints/20))*20
    o7=soft_step(sin_osc(seed/278.5,0,maxPoints/20))*20
    o8=soft_step(sin_osc(seed/491.5,0,maxPoints/20))*20

    disrupt=0
    for i in range(maxPoints):
        h=i/maxPoints*math.pi*2
        x = math.sin(-h*o1)
        y = math.cos(-h*o1)


        if i>o3 and i<o4:
            if int(i/8+soft_step(o9))%2==0:
                disrupt+=1
                x=x*o2/6
                y=y*o2/6
        if i>o5 and i<o6:
            disrupt+=1
            x=x+o2/20
        if i>o6 and i<o3:
            disrupt+=1
            x=x-o2/10
        if i>o4 and i<o5:
            if int(i/16+soft_step(o1))%2==0:
                disrupt+=1
                x=x*o2/3*i/maxPoints
                y=y*o2/3*i/maxPoints
        if i>o7 and i<o8:
            disrupt+=1
            y=y-o2/10
        if i>o8 and i<o7:
            if int(i/12+soft_step(o2))%2==0:
                disrupt+=1
                x=x*o9/16
                y=y*o9/16
        if i>maxPoints-10 and disrupt==0 and (i)%2==0:
            x=random.randint(0,10)/10
            y=random.randint(0,10)/10
        else:
            x=x/2+.5
            y=y/2+.5
        points.append((x,y,1))


    return points, "SunshineGears"
#=============================================================================== 70
# 11segments

segmentDir=[]
segmentIndex=0
for i in range(1000):
    segmentDir.append(random.randint(0,100))
def generateL70(seed, maxPoints):
    global segmentDir, segmentIndex
    segmentIndex=int(seed*3)%int(maxPoints)
    segmentDir[segmentIndex]=random.randint(0,100)
    points = []
    xs=0
    ys=0
    charPos=0
    for i in range(int(maxPoints/40)):
        si=(i)%100
        if xs==0:
            if ys==0:
                s=segmentDir[si]%5
                if s==0:
                    ys=1
                elif s==1:
                    ys=-1
                elif s==2:
                    xs=1
                elif s==3:
                    xs=1
                    ys=1
                elif s==4:
                    xs=1
                    ys=-1
            else:
                s=segmentDir[si]%3
                if s==0:
                    xs=1
                elif s==1:
                    ys=0
                elif s==2:
                    xs=1
                    ys=0

        else: #xs==1
            if ys==0:
                s=segmentDir[si]%8
                if s==0:
                    ys=1
                elif s==1:
                    ys=-1
                elif s==2:
                    xs=0
                elif s==3:
                    xs=0
                    ys=1
                elif s==4:
                    xs=0
                    ys=-1
                else:
                    xs=0
                    ys=0
                    charPos+=1
            else:
                s=segmentDir[si]%3
                if s==0:
                    xs=0
                elif s==1:
                    ys=0
                elif s==2:
                    xs=0
                    ys=0
        for j in range(40):
            points.append((xs*.15+charPos*.15,ys*.1,1))

    return points, "11Segments"
#=============================================================================== 69
# 7segments
segmentDir7=[]
segmentIndex7=0
for i in range(1000):
    segmentDir7.append(random.randint(0,100))

def generateL69(seed, maxPoints):
    global segmentDir7, segmentIndex7
    segmentIndex7=int(seed*3)%int(maxPoints)
    segmentDir7[segmentIndex7]=random.randint(0,100)
    points = []
    xs=0
    ys=0
    charPos=0
    for i in range(int(maxPoints/10)):
        si=(i)%100
        if segmentDir7[si] >70:
            if xs==0:
                if ys==0:
                    s=segmentDir7[si]%4
                    if s==0:
                        ys=1
                    elif s==1:
                        ys=-1
                    else:
                        xs=1
                else:
                    if segmentDir7[si]%2==0:
                        xs=1
                    else:
                        ys=0
            elif xs==1:
                if ys==0:
                    s=segmentDir7[si]%6
                    if s==0:
                        ys=1
                    elif s==1:
                        ys=-1
                    elif s==2:
                        xs=0
                    elif segmentDir7[si]>80:
                        xs=0
                        ys=0
                        charPos+=1
                else:
                    if segmentDir7[si]%2==0:
                        xs=0
                    else:
                        ys=0
        for j in range(10):
            points.append((xs*.075+charPos*.1,ys*.075,1))


    # Extract x and y values using list comprehensions
    x_values = [point[0] for point in points]
    y_values = [point[1] for point in points]

    # Find the minimum and maximum for x and y
    min_x = min(x_values)
    max_x = max(x_values)
    min_y = min(y_values)
    max_y = max(y_values)

    centerX=(min_x-max_x)/2

    pointsCorrected=[]

    pointsCorrected.append((0,0.5,1))
    pointsCorrected.append((x_values[0]+centerX+.5,.5,1))
    for i in range(maxPoints-5):
        pointsCorrected.append((x_values[i]+centerX+.5,y_values[i]+.5,1))

    pointsCorrected.append((x_values[maxPoints-4]+centerX+.5,y_values[maxPoints-4]+.5,1))
    pointsCorrected.append((x_values[maxPoints-4]+centerX+.5,.5,1))
    pointsCorrected.append((1,.5,1))

    return pointsCorrected, "7Segments"


#=============================================================================== 68
# gearCake
def generateL68(seed, maxPoints):
    seed/=128
    points = []

    b =(sin_osc(seed/81.61,-500,500)+sin_osc(seed/629.2,-500,500)+sin_osc(seed/5867,-500,500))/500*math.pi
    c1=soft_step((abs(sin_osc(seed/88.61,-500,500)+sin_osc(seed/869.2,-500,500)+sin_osc(seed/6857,-500,500))%1000)/100)*100
    c2=soft_step((abs(sin_osc(seed/83.52,-500,500)+sin_osc(seed/971.1,-500,500)+sin_osc(seed/6353,-500,500))%1000)/200)*200
    c3=soft_step((abs(sin_osc(seed/81.43,-500,500)+sin_osc(seed/629.3,-500,500)+sin_osc(seed/6254,-500,500))%1000)/300)*300
    c4=abs(sin_osc(seed/85.61,-500,500)+sin_osc(seed/939.2,-500,500)+sin_osc(seed/5657,-500,500))
    c5=abs(sin_osc(seed/84.52,-500,500)+sin_osc(seed/841.1,-500,500)+sin_osc(seed/5753,-500,500))
    c6=sin_osc(seed/83.43,-500,500)+sin_osc(seed/759.3,-500,500)+sin_osc(seed/5854,-500,500)
    u =(sin_osc(seed/61.61,33,500)+sin_osc(seed/521.8,33,500)+sin_osc(seed/7851,33,500))/150

    star=0
    for i in range(maxPoints):
        a = b + i/maxPoints*2*math.pi*u
        s=1
        if (i>c1):
            a+=math.pi
            s*=sin_osc(seed/124,.5,1)
        if (i>c2):
            a+=math.pi
            s*=sin_osc(seed/214,.5,1)
        if (i>c3):
            a+=math.pi
            s*=sin_osc(seed/304,.5,1)
        if (i>c4):
            a+=math.pi
            s*=sin_osc(seed/494,.5,1)
        if (i>c5):
            a+=math.pi
            s*=sin_osc(seed/584,.5,1)
        if (i>c6):
            a+=math.pi
            s*=sin_osc(seed/674,.5,1)
        if (s<.2):
            s=0
        if (s<.3):
            s=.5

        if (i>c1 and i<c2 and int(i/3)%2==0 and star<20):
            s=s*sin_osc(seed/56.4,.5,1.2)
            star+=1
        if (i>c3 and i<c4 and int(i/4)%2==0 and star<40):
            s=s*sin_osc(seed/62.4,.5,1.2)
            star+=1
        if (i>c5 and i<c6 and int(i/5)%2==0 and star<60):
            s=s*sin_osc(seed/264,.5,1.2)
            star+=1
        x=math.sin(a)*s
        y=math.cos(a)*s

        x=x/2+.5
        y=y/2+.5
        points.append((x,y,1))
    return points, "GearCake"
#=============================================================================== 67
# centralquads
def generateL67(seed, maxPoints):
    seed=seed/24
    points = []

    c = int(sin_osc(seed/143.4,3,50))

    for i in range(maxPoints):
        s=seed

        separations = soft_step(sin_osc(s/3261,2,16))
        separator = int(i/maxPoints*separations)/separations

        amount = soft_step(sin_osc(s/1000,1,30))
        qCount = int(i/maxPoints*4*amount)/2*math.pi
        qCount+= sin_osc(soft_step((seed+i/10)/1461),-math.pi,math.pi)

        x = math.sin(qCount)
        y = math.cos(qCount)

        scale = soft_step(sin_osc(soft_step(seed/1234+i/120),1,50))/10
        scale=scale/sin_osc(seed/1735,1,10)

        xShift = sin_osc((separator-separations/2)*6+189,-.4,.4)
        yShift = sin_osc((separator-separations/2)*5+375,-.4,.4)

        x=(x*scale)/2+.5+xShift
        y=(y*scale)/2+.5+yShift

        if (int(i/24)%c==0):
            x=0.5
            y=0.5
        points.append((x,y,1))
    return points, "CentralQuads"
#=============================================================================== 66
# quads
def generateL66(seed, maxPoints):
    seed=seed/24
    points = []

    border = soft_step(sin_osc(seed/768,1,maxPoints)/10)*10

    for i in range(maxPoints):
        s=seed
        if (i>border):
            s = -seed
        separations = soft_step(sin_osc(s/3261,2,32))
        separator = int(i/maxPoints*separations)/separations

        amount = soft_step(sin_osc(s/1000,1,10))
        hexCount = int(i/maxPoints*6*amount)/2*math.pi
        hexCount += (soft_step(sin_osc(s/465.7,1,6))+soft_step(sin_osc(s/3631,1,6)))/2*math.pi

        hexCount+=soft_step(soft_step(separator*sin_osc(s/773913,1,12))*math.pi/2)
        x = math.sin(hexCount)
        y = math.cos(hexCount)
        scale = soft_step(sin_osc(s/3349+separator*2,1,12))/13
        x=(x*scale)/2+.5
        y=(y*scale)/2+.5

        points.append((x,y,1))
    return points, "Quads"
#=============================================================================== 65
# hexagons
def generateL65(seed, maxPoints):

    seed=seed/16
    points = []

    border = sin_osc(seed/578,1,maxPoints)

    for i in range(maxPoints):
        s=seed
        if (i>border):
            s = -seed

        separations = soft_step(sin_osc(s/3561,2,24))
        separator = int(i/maxPoints*separations)/separations

        amount = soft_step(sin_osc(s/1000,1,10))
        hexCount = int(i/maxPoints*6*amount)/6*math.pi*2
        hexCount += (soft_step(sin_osc(s/435.7,1,6))+soft_step(sin_osc(s/3637,1,6)))*math.pi/6

        hexCount+=soft_step(soft_step(separator*sin_osc(s/778913,1,12))*math.pi/3)
        x = math.sin(hexCount)
        y = math.cos(hexCount)
        scale = soft_step(sin_osc(s/3339+separator*2,1,12))/13
        x=(x*scale)/2+.5
        y=(y*scale)/2+.5

        points.append((x,y,1))
    return points, "Hexagons"



#=============================================================================== 64
# sworm

swormCoords = []
for i in range(100):
    swormCoords.append((.5,.5))#(random.randint(0,10)/10,random.randint(0,10)/10))

def generateL64(seed, maxPoints):


    for i in range(100):    #get the swormPoint to manipulate
        xs,ys = swormCoords[i]

        filter = sin_osc((seed)/68736,100,100000)
        xs=(xs*filter+.5)/(filter+1) # apply center gravity
        ys=(ys*filter+.5)/(filter+1)

        filter = sin_osc((seed+i)/287.36,50,5000)
        xs=((xs*filter+random.randint(0,100)/100)/(filter+1))  # apply noise
        ys=((ys*filter+random.randint(0,100)/100)/(filter+1))


        for j in range(100): #all the other sworm points


            if j!=i:
                thresh = sin_osc((seed+i*100)/3538,.01,.001)
                x,y = swormCoords[i]
                d = distance(xs,ys,x,y)

                if d > thresh: # get closer
                    filter = abs(j-i)
                    xs = (xs*filter + x)/(filter+1)
                    ys = (ys*filter + y)/(filter+1)


                else: # get further away
                    filter = sin_osc((seed)/2000+(j/10),1,500)
                    x,y = distancePoint(xs, ys, x, y, thresh)
                    xs = (xs*filter + x)/(filter+1)
                    ys = (ys*filter + y)/(filter+1)

                swormCoords[i] = (xs,ys)

    points = []
    for i in range(maxPoints):
        j = i//10
        x,y = swormCoords[j]
        points.append((x,y))

    return points, "Sworm"

def distancePoint(xa, ya, xb, yb, d):
    # Calculate total distance between points
    total_distance = math.sqrt((xa - xb)**2 + (ya - yb)**2)

    # If points are the same, we can't determine direction
    if total_distance == 0:
        return xb, yb

    # Calculate the ratio of the desired distance to total distance
    ratio = d / total_distance

    # Calculate direction vector
    dx = xa - xb
    dy = ya - yb

    # Calculate new point coordinates
    x = xb + (dx * ratio)
    y = yb + (dy * ratio)

    return x, y
#=============================================================================== 63
# cube
def generateL63(seed, maxPoints):

    sec = seed / 2000
    points = []
    # Example coordinates
    coords = [-1,-1,-1],[1,-1,-1],[1,1,-1],[-1,1,-1],[-1,-1,-1],[-1,-1,1],[1,-1,1],[1,1,1],[-1,1,1],[-1,-1,1],[1,-1,1],[1,-1,-1],[1,1,-1],[1,1,1],[-1,1,1],[-1,1,-1]
    # Fill the array

    so = sin_osc(sec/361,.1,.6)
    for i in range(334):
        p = round(i/18)
        p = p%len(coords)
        # Rotation angles (in radians)
        theta_x = sec / 1.47
        theta_y = sec / 1.66
        theta_z = sec / 1.31  #
        # Rotate coordinates
        rotated_coords = rotate_z(rotate_y(rotate_x(coords[p], theta_x), theta_y), theta_z)
        # Project onto 2D
        x,y = project_2d(rotated_coords)

        points.append((x*so+.4,y*so+.4,1))
    xo = sin_osc(sec/124,-.3,.3)
    yo = sin_osc(sec/171,-.3,.3)
    so = sin_osc(sec/271,.1,.6)

    for i in range(334):
        p = round(i/17)
        p = p%len(coords)
        # Rotation angles (in radians)
        theta_x = sec / 1.37
        theta_y = sec / 1.56
        theta_z = sec / 1.81  #
        # Rotate coordinates
        rotated_coords = rotate_z(rotate_y(rotate_x(coords[p], theta_x), theta_y), theta_z)
        # Project onto 2D
        x,y = project_2d(rotated_coords)

        points.append((x*so+.5+xo,y*so+.5+yo,1))

    xo = sin_osc(sec/317,-.3,.3)
    yo = sin_osc(sec/189,-.3,.3)
    so = sin_osc(sec/711,.1,.6)
    for i in range(334):
        p = round(i/17)
        p = p%len(coords)
        # Rotation angles (in radians)
        theta_x = sec / 1.37
        theta_y = sec / 1.56
        theta_z = sec / 1.81  #
        # Rotate coordinates
        rotated_coords = rotate_z(rotate_y(rotate_x(coords[p], theta_x), theta_y), theta_z)
        # Project onto 2D
        x,y = project_2d(rotated_coords)

        points.append((x*so+.5+xo,y*so+.5+yo,1))

    return points, "TwoCubes"

#=============================================================================== 62
rLineAngles = []
rLineTargets = []
rLineLength = []
rLineLengthTargets = []
for i in range(10):
    rLineAngles.append(random.randint(-314,314)/100)
    rLineTargets.append(rLineAngles[i])
    rLineLength.append(random.randint(1,20)/10)
    rLineLengthTargets.append(rLineLength[i])

def generateL62(seed, maxPoints):

    global rLineAngles,rLineTargets
    points = []

    for i in range(10):
        if random.randint(0,1000)==1:
            rLineTargets[i]=(random.randint(-314,314)/100)
        if random.randint(0,5000)==1:
            rLineLengthTargets[i]=(random.randint(1,20)/10)

        a=rLineAngles[i]
        b=rLineTargets[i]
        rLineAngles[i]=((a*100+b)/101)

        a=rLineLength[i]
        b=rLineLengthTargets[i]
        rLineLength[i]=((a*100+b)/101)

    for i in range(maxPoints):
        j = int(i/100)
        l = (j+int(sin_osc(seed/89856,0,10)))%10
        k = i%100
        x = j/10
        y = .5
        x+= (math.sin(seed/589.64+j*5+rLineAngles[l])-1)/1000*(k-50)*rLineLength[l]
        y+= (math.cos(seed/589.64+j*5+rLineAngles[l])-1)/1000*(k-50)*rLineLength[l]

        points.append((x,y))
    offset = (int(sin_osc(seed/1173,100,maxPoints*1.5))+int(sin_osc(seed/1751,100,maxPoints*1.5))+int(sin_osc(seed/1724,100,maxPoints*1.5))) % len(points)
    offset = maxPoints-offset
    reordered_points = points[offset:] + points[:offset]
    return reordered_points,"Clks"

#=============================================================================== 61

tohuwabohuIndex = 0

def generateL61(seed, maxPoints):

    global tohuwabohuIndex
    seed/=100
    points = []

    tohuwabohuIndex+=.2

    if int(tohuwabohuIndex)>=maxPoints:
        tohuwabohuIndex=0
    counter=0
    for i in range(10):
        x = sin_osc(sin_osc(seed/117431+i,0,20),0,1)
        y = sin_osc(sin_osc(seed/115834+i,0,20),0,1)

        for j in range(10):
            xx = x+sin_osc(sin_osc(seed/31.731+j+i/2,0,3)+sin_osc(seed/1137.31+j+i/20,0,3),-.2,.2)
            yy = y+sin_osc(sin_osc(seed/27.534+j+i/2,0,3)+sin_osc(seed/177.31+j+i/20,0,3),-.2,.2)

            for  k in range (10):

                counter+=1;
                comp = int(sin_osc(seed/89.886,3,300))
                xxx=0
                yyy=0
                if (int(tohuwabohuIndex/10)%comp == int(counter/10)%comp):
                    xxx=sin_osc(seed/1.8734+j,-.2,.2)
                    yyx=sin_osc(seed/1.7831+j,-.2,.2)

                points.append((xx+xxx,yy+yyy))

    return points, "Tohuwabohu"

#=============================================================================== 60
def generateL60(seed, maxPoints):

    p2d, name = generateL10(seed/2, maxPoints)
    ax = seed/2234.6
    ay = seed/2345.6

    points = []
    for i in range(200):
        x = sin_osc(seed/15678+i/30,0,2) + sin_osc(seed/11913+i/12,0,2)
        y = sin_osc(seed/11678+i/31,0,2) + sin_osc(seed/17983+i/13,0,2)
        cx,cy = project_to_sphere(x,y,ax,ay)
        cx = cx/2+.5
        cy = cy/2+.5
        for i in range(5):
            points.append((cx,cy))

    return points, "Wobblob"
#=============================================================================== 59
bubbleCenters = []
bubbleDirections = []

for i in range(10):
    bubbleCenters.append(((random.randint(0,20)-10)/10,((random.randint(0,20)-10)/10)))
    bubbleDirections.append((random.randint(0,1)-.5,random.randint(0,1)-.5))

bubbleSeed = 0
def generateL59(seed, maxPoints):

    global bubbleCenters, bubbleDirections, bubbleSeed
    sDif = max(seed-bubbleSeed,.1)*50000
    bubbleSeed = seed
    ax = seed/12346
    ay = seed/23456

    centers = []
    for i in range(10):
        xf, yf = bubbleCenters[i]
        xd, yd = bubbleDirections[i]
        xf+=xd/sDif
        yf+=yd/sDif
        bubbleCenters[i] = (xf,yf)
        cx,cy = project_to_sphere(xf,yf,ax,ay)
        centers.append((cx/2+.5,cy/2+.5))

    points = []

    for i in range(maxPoints):
        j = int (i/100)
        cx, cy = centers[j]
        c = i%100
        x=cx
        y=cy

        t1 = ((cx)*100)%100
        t2 = ((cy)*100)%100

        if (c>=t1 and c<=t2):
            x = math.sin(c/25*math.pi)/10+cx
            y = math.cos(c/25*math.pi)/10+cy

        points.append((x,y))

    return points, "PacMan"

#=============================================================================== 58
def generateL58(seed, maxPoints):

    p2d, name = generateL10(seed/2, maxPoints)

    points = []
    for i in range(500):

        x,y=project_to_sphere(p2d[i][0]-.5,p2d[i][1]-.5,p2d[999][1]*2,-p2d[999][0]*2)
        x=x/3+.5
        y=y/3+.5
        for j in range(2):
            points.append((x,y))

    return points, "SnakGlob"

#=============================================================================== 57
def generateL57(seed, maxPoints):

    ax = -(seed)/23871
    ay = -(seed)/17132
    p2d, name = generateL46(seed/2, maxPoints)

    points = []
    s = sin_osc(seed/8865,1,6)
    s2 = sin_osc(seed/3861,.25,1.25)
    for i in range(500):

        x,y=project_to_sphere(p2d[i][0]*s,p2d[i][1]*s2-.5,ax,ay)

        x=x/2+.5
        y=y/2+.5
        for j in range(2):
            points.append((x,y))

    return points, "GridGlob"

#=============================================================================== 56
def generateL56(seed, maxPoints):

    ax = -(seed)/23871
    ay = -(seed)/17132
    p2d, name = generateL49(seed/2, maxPoints)

    points = []
    for i in range(500):

        x,y=project_to_sphere(p2d[i][0]*2-1,p2d[i][1]*2-1,ax,ay)

        x=x/2+.5
        y=y/2+.5
        for j in range(2):
            points.append((x,y))

    return points, "BubbleShift"

#=============================================================================== 55
def generateL55(seed, maxPoints):

    ax = -(seed)/23871
    ay = -(seed)/17132
    p2d, name = generateL30(seed/2, maxPoints)

    points = []
    for i in range(500):
        x,y=project_to_sphere(p2d[i][0]*2-1,p2d[i][1]*2-1,ax,ay)
        for j in range(2):
            points.append((x,y))

    return points, "BubbleZag"

#=============================================================================== 54


def generateL54(seed, maxPoints):

    ax = (seed)/3810
    ay = (seed)/13170

    o1 = sin_osc(seed/12465,100,2000)
    o2 = sin_osc(seed/32475,100,2000)
    o3 = sin_osc(seed/72421,0,1000)

    s1 = sin_osc(seed/1214615,5,20)
    s2 = sin_osc(seed/1714165,5,20)

    points = []
    for i in range(250):

        x = i/250 - 1
        y = saw_osc(i/(o1))*2-1

        if (i>o3):
            j=i-500
            y = j/250 - 1
            x = saw_osc(j/(o2))*2-1

        x=int(x*s1)/s1
        y=int(y*s2)/s2

        x2,y2=project_to_sphere(x,y,ax,ay)
        x2=max(min(x2/2+.5,1),0)
        y2=max(min(y2/2+.5,1),0)

        for k in range(4):
            points.append((x2,y2))

    return points, "RectiGlobe"

#=============================================================================== 53

def generateL53(seed, maxPoints):


    points = []

    seed/=100
    ax = (seed)/12103
    ay = (seed)/13170
    for i in range(500):

        x1 = int((math.sin((i/5+seed)/13.4)/2 +  math.sin((i/6+seed)/3.15)/2 + math.sin((i/7+seed)/7.14)/2 )*3)/3
        y1 = int((math.sin((i/5+seed)/13.6)/2 +  math.sin((i/6+seed)/3.85)/2 + math.sin((i/7+seed)/7.94)/2 )*3)/3

        x2, y2 = project_to_sphere(x1-.5,y1-.5,ax,ay)
        x2=max(min(x2/2+.5,1),0)
        y2=max(min(y2/2+.5,1),0)

        points.append((x2,y2))
        points.append((x2,y2))

    return points, "FailedRotation"

#=============================================================================== 52

def generateL52(seed, maxPoints):

    seed /=500
    points = []
    d = 1000/7
    for i in range(maxPoints):
        x = saw_osc((i+seed*.0315)/d) * saw_osc((i+seed*.315)/d)  * saw_osc((i+seed*3.15)/d)  * saw_osc((i+seed*7.15)/d)
        y = saw_osc((i+seed*.0134)/d) * saw_osc((i+seed*.134)/d)  * saw_osc((i+seed*1.34)/d)  * saw_osc((i+seed*3.34)/d)
        points.append((x,y))


    return points, "Stripes"

#=============================================================================== 51

def generateL51(seed, maxPoints):


    points = []

    cx1 = sin_osc(seed/3899,0.25,.75)
    cy1 = sin_osc(seed/3989,0.25,.75)
    rx = sin_osc(seed/4763,.1,.4)
    ry = sin_osc(seed/4533,.1,.4)
    ll = sin_osc(seed/4816,80,320)
    sl = sin_osc(seed/5816,0,30)

    cx0 = sin_osc(seed/2599,0.25,.75)
    cy0 = sin_osc(seed/2389,0.25,.75)

    cx2 = sin_osc(seed/2119,0.25,.75)
    cy2 = sin_osc(seed/2749,0.25,.75)

    db1 = sin_osc(seed/1467,-50,900)
    de1 = sin_osc(seed/2361,100,1050)

    db2 = sin_osc(seed/1561,-50,900)
    de2 = sin_osc(seed/2663,100,1050)

    if (db1>de1) and (db2>de2):
        m=db1
        db1=de1
        de1=m


    for i in range(maxPoints):
        x = cx1 + math.sin((i+sl)/ll+sin_osc(seed/976,0,.5))*rx
        y = cy1 + math.cos((i+sl)/ll)*ry

        if (int(i/40)%2==0 and i>db1 and i<de1):
            x=(x+cx0)/2
            y=(y+cy0)/2

        if (int((i)/40+sl)%2==0 and i>db2 and i<de2):
            x=(x+cx2)/2
            y=(y+cy2)/2

        points.append((x,y))

    return points, "Vase"

#=============================================================================== 50

def generateL50(seed,maxPoints):

    points = []
    seed = seed/11.5
    start1 = sin_osc(seed/88.5,0,maxPoints)
    end1   = sin_osc(seed/67.3,0,maxPoints)
    o1 = sin_osc(seed/997,.1,.8)
    si1 = sin_osc(seed/976,0,20)
    start2 = sin_osc(seed/58.5,0,maxPoints)
    end2   = sin_osc(seed/97.3,0,maxPoints)
    o2 = sin_osc(seed/797,.1,.8)
    si2 = sin_osc(seed/676,0,20)
    start3 = sin_osc(seed/78.5,0,maxPoints)
    end3   = sin_osc(seed/197.3,0,maxPoints)
    o3 = sin_osc(seed/597,.1,.8)
    si3 = sin_osc(seed/376,0,20)

    if (start1>end1  and start3>end3):
        b=start1
        start1 = end1
        end1 = b

    for i in range(maxPoints):
        x = 0
        y = 0
        if i < maxPoints/4:  # First segment (bottom)
            y = 0
            x = i/(maxPoints/4)
        elif i < maxPoints/2:  # Second segment (right side)
            x = 1
            y = (i-maxPoints/4)/(maxPoints/4)
        elif i < 3*maxPoints/4:  # Third segment (top)
            y = 1
            x = 1 - (i-maxPoints/2)/(maxPoints/4)
        else:  # Fourth segment (left side)
            x = 0
            y = 1 - (i-3*maxPoints/4)/(maxPoints/4)

        x-=.5
        y-=.5

        if i>start1 and i<end1 and int((i+si1)/10)%2==0:
            x*=o1
            y*=o1
        if i>start2 and i<end2 and int((i+si2)/10)%2==0:
            x*=o2
            y*=o2
        if i>start3 and i<end3 and int((i+si3)/10)%2==0:
            x*=o3
            y*=o3

        x+=.5
        y+=.5

        points.append((x,y))
        offset = int(seed/11.5) % len(points)
        offset = maxPoints-offset
        reordered_points = points[offset:] + points[:offset]

    return reordered_points,"DepthSquare"


#=============================================================================== 49

shiftCoords1 = []
shiftCoords2 = []

def fill_shiftCoords():
    global shiftCoords1, shiftCoords2
    for i in range(1000):
        value = (i / 1000, i / 1000)  # Create the tuple
        shiftCoords1.append(value)   # Append to shiftCoords1
        shiftCoords2.append(value)   # Append to shiftCoords2

fill_shiftCoords()

def generateL49(seed, maxPoints):

    global shiftCoords1, shiftCoords2
    # reset

    if (random.randint(1, 300)==1):
        r_start = random.randint(-400,1000)
        r_end = random.randint(0,400)
        r_end = r_start+r_end
        r_start = max(0,min(999,r_start))
        r_end = max(0,min(999,r_end))

        r= random.randint(0,3)

        for i in range(r_start, r_end):
            if r==0:
                shiftCoords2[i] = (i/1000,i/1000)
            elif r==1:
                shiftCoords2[i] = (1-i/1000,i/1000)
            elif r==2:
                shiftCoords2[i] = (i/1000,1-i/1000)
            else:
                shiftCoords2[i] = (1-i/1000,1-i/1000)


    # shift
    if (random.randint(1, 300)==1):
        r_start = random.randint(-200,1000)
        r_end = random.randint(0,200)
        r_end = r_start+r_end
        r_start = max(0,min(979,r_start))
        r_end = max(0,min(979,r_end))
        r_x = 0
        r_y = 0

        if (random.randint(0,1)==0):
            r_x = (random.randint(0,4)-2)/10
        else:
            r_y = (random.randint(0,4)-2)/10

        for i in range(r_start, r_end, 20):
            for j in range(10):
                xa,ya =shiftCoords1[i+j]
                xa += r_x
                ya += r_y
                xa = max(0,min(1,xa))
                ya = max(0,min(1,ya))
                shiftCoords2[i+j] = xa,ya

    # smoothen
    for i in range(1000):
        xa,ya =shiftCoords1[i]
        xb,yb =shiftCoords2[i]
        shiftCoords1[i] = ((xa*10000+xb*500+50)/10600,(ya*10000+yb*500+50)/10600)

    points = []

    for i in range(maxPoints):
        points.append(shiftCoords1[i])

    return points, "Shifter"


#=============================================================================== 48

crystalCoords1 = []
crystalCoords2 = []

def fill_crystalCoords():
    global crystalCoords1, crystalCoords2
    crystalCoords1 = [(random.randint(0, 4)/4, random.randint(0, 4)/4) for _ in range(11)]
    crystalCoords2 = crystalCoords1[:]

def replace_coordinate(index):
    global crystalCoords1, crystalCoords2
    if 0 <= index < len(crystalCoords1):
        crystalCoords2[index] = (random.randint(0, 4)/4, random.randint(0, 4)/4)
fill_crystalCoords()

def generateL48(seed, maxPoints):

    global crystalCoords1, crystalCoords2
    if (random.randint(1, 300)==1):
        replace_coordinate(random.randint(0, 10))
    for i in range(11):
        xa,ya =crystalCoords1[i]
        xb,yb =crystalCoords2[i]
        crystalCoords1[i] = ((xa*1000+xb)/1001,(ya*1000+yb)/1001)

    points = []

    s= (sin_osc(seed/4894,75,125))
    for i in range(250):
        x1,y1 = crystalCoords1[int(i/25)]
        x2,y2 = crystalCoords1[int(i/25)+1]
        l1 = (i%s)/s
        l2 = 1-l1
        x=x1*l1 + x2*l2
        y=y1*l1 + y2*l2
        for j in range(4):
            points.append((x,y))
    # Reorder points based on seed
    offset = int(seed/14.5) % len(points)
    reordered_points = points[offset:] + points[:offset]
    return reordered_points, "Crystal"


#=============================================================================== 47


def generateL47(seed, maxPoints):

    seed = seed/3.24
    points = []
    for i in range(maxPoints):
        x = 1-(int(i/40)%5)/5
        y = 1-(int(i/200)/5)

        d = distance(x,y,.5,.5)

        m = sin_osc(d*5+seed/945,5,50)

        x=x+math.sin(i/5+seed/245.6)/m
        y=y+math.cos(i/5+seed/245.6)/m

        points.append((x,y))
    # Reorder points based on seed
    offset = int(sin_osc(seed/1545,0,500)) % len(points)
    reordered_points = points[offset:] + points[:offset]
    return reordered_points, "Cirqules"

#=============================================================================== 46
def generateL46(seed, maxPoints):

    seed = seed/3.65
    points = []
    for i in range(maxPoints):
        x = int(i/sin_osc(seed/186,200,201))/sin_osc(seed/195,4,6)

        x = x+ (int(i/10)%2)/5 + sin_osc(int(seed/6718 + i/200 + sin_osc(seed/1985,0,1)),0,.1)
        y = (i%200)/sin_osc(seed/48,199,201) + sin_osc(seed/568 + i/46,0,.1) + sin_osc(seed/671 + i/39,0,.1) + sin_osc(seed/361 + i/75,0,.1)  + sin_osc(seed/389 + i/31.5,0,.1)
        points.append((x,y))

    return points, "SquareGrid"


#=============================================================================== 45

def generateL45(seed, maxPoints):

    seed = seed/9.87
    points = []
    for i in range(maxPoints):
        x=0
        y=0
        l1 = sin_osc(seed/123, 50, 149)
        l2 = sin_osc(seed/245, 150, 249)
        l3 = sin_osc(seed/789, 250, 349)
        l4 = sin_osc(seed/835, 350, 449)
        if (i<l1):
            x=i/l1
            y=0
        elif (i>=l1 and i<l2):
            x=1
            y=(i-l1)/100
        elif (i>=l2 and i<l3):
            x=1-(i-l2)/100
            y=1
        elif (i>=l3 and i<l4):
            x=0
            y=1-(i-l3)/100
        else:
            j = i/6
            if j%2 == 0:
                x=0
            else:
                x=1
            y = sin_osc((seed/862+i/sin_osc(seed/992,100,200)),0,1)+x*sin_osc((seed+i)/9986,-.01,.01)
        x=max(-.05,min(x,1.05))
        y=max(-.05,min(y,1.05))
        points.append((x,y))

    # Reorder points based on seed
    offset = int(seed/100) % len(points)
    reordered_points = points[offset:] + points[:offset]

    return reordered_points, "Gradient"




#=============================================================================== 44
neuronBuffer = np.zeros((1000, 2)) # contains position values in the format (x,y)
neuronIndex = 0
neuronX = 50
neuronY = 50

def generateL44(seed, maxPoints):

    global neuronBuffer, neuronIndex, neuronX, neuronY
    random.seed(seed)

    oldIndex = int(neuronIndex)
    neuronIndex = (neuronIndex+.1) % maxPoints
    position = neuronBuffer[oldIndex]

    direction = random.randint(1, 4)
    if direction == 1:
        # move right (x+1)
        position[0] = min(max(position[0] + 1, 0), 99)
    elif direction == 2:
        # move down (y+1)
        position[1] = min(max(position[1] + 1, 0), 99)
    elif direction == 3:
        # move left (x-1)
        position[0] = min(max(position[0] - 1, 0), 99)
    elif direction == 4:
        # move up (y-1)
        position[1] = min(max(position[1] - 1, 0), 99)

    neuronBuffer[int(neuronIndex)] = position

    points = []
    for i in range(maxPoints):
        shifted_index = (int(neuronIndex) + 1 + i) % maxPoints
        point = (neuronBuffer[shifted_index][0] / 100, neuronBuffer[shifted_index][1] / 100)
        points.append(point)

    return points, "LostNeuron"

#=============================================================================== 43

def generateL43(seed, maxPoints):

    points = []
    divider = sin_osc(seed/1756,15,30)

    for i in range(maxPoints):
        x=0
        y=0
        if i>sin_osc(seed/1123,-10,maxPoints+10):
            shift = saw_osc(seed/19713)+i
            x = squ_osc(shift/divider)*sin_osc(seed/6126,.25,.5)+sin_osc(seed/5176,0,.5)
            y = squ_osc(shift/divider+min(saw_osc(seed/1899)*4,1))*sin_osc(seed/7166,.25,.5)+sin_osc(seed/4193,0,.5)
        elif i>sin_osc(seed/1723,-10,maxPoints+10):
            shift = saw_osc(seed/17153)+i
            x = squ_osc(shift/divider)*sin_osc(seed/7124,.25,.5)+sin_osc(seed/6182,0,.5)
            y = squ_osc(shift/divider+min(saw_osc(seed/1769)*4,1))*sin_osc(seed/5176,.25,.5)+sin_osc(seed/4173,0,.5)
        else :
            shift = saw_osc(seed/13153)+i
            x = squ_osc(shift/divider)*sin_osc(seed/5124,.25,.5)+sin_osc(seed/4161,0,.5)
            y = squ_osc(shift/divider+min(saw_osc(seed/1261)*4,1))*sin_osc(seed/5173,.25,.5)+sin_osc(seed/2191,0,.5)

        x = max(min(x,1),0)
        y = max(min(y,1),0)
        points.append((x,y))

    return points, "Rectangular"

def squ_osc(value): #oscillates between 0 and 1 in flattened sawtooth
    returnValue = 0
    value = value%4
    if value<1:
        returnValue = value
    if value >=1 and value <2:
        returnValue = 1
    if value>=2 and value <3:
        returnValue = 3-value
    if value >= 3:
        returnValue = 0
    return returnValue


#=============================================================================== 42
def generateL42(seed, maxPoints):

    points = []
    o1 = (math.sin(seed/1819)+1.5)*10
    o2 = (math.cos(seed/718)+1.5)*10
    o3 = (math.cos(seed/123456)+2)*100


    for i in range(maxPoints):
        x=0
        y=0
        if i>sin_osc(seed/1123,-10,maxPoints+10):
            x=sin_osc(seed/1130.15,0,1)
            if math.sin(i/(o1+o2)+o3)>0:
                x=sin_osc(seed/1241.16,0,1)
            y=sin_osc(seed/1152.3,0,1)
            if math.cos(i/(o1+o2)+o3)>0:
                y=sin_osc(seed/1263.47,0,1)
            elif i>sin_osc(seed/1973.1,-10,maxPoints+10):
                x=sin_osc(seed/01135.15,0,1)
                if math.sin(i/(o1+o2)+o3)>0:
                    x=sin_osc(seed/1346.16,0,1)
                y=sin_osc(seed/1258.3,0,1)
                if math.cos(i/(o1+o2)+o3)>0:
                    y=sin_osc(seed/1367.47,0,1)
            else:
                x=sin_osc(seed/01635.15,0,1)
                if math.sin(i/(o1+o2)+o3)>0:
                    x=sin_osc(seed/1846.16,0,1)
                y=sin_osc(seed/1758.3,0,1)
                if math.cos(i/(o1+o2)+o3)>0:
                    y=sin_osc(seed/1967.47,0,1)
        points.append((x,y))

    return points, "FalseRectangular"

#=============================================================================== 41
# RectShift
pSegSeed = 0
def generateL41(seed, maxPoints):

    global pSegSeed,segXPoints,segYPoints,segStart,segEnd,segVeloX,segVeloY,segCount,segCount
    random.seed(seed)
    step = seed-pSegSeed
    pSegSeed = seed

    segCount -= 1
    if (segCount <= 0):
        if random.randint(0, 6) == 0:
            # reset some points to the position of the first in this sequence
            startI = random.randint(0, maxPoints-1)
            endI = random.randint(startI, maxPoints-1)
            for i in range(startI,endI):
                segXPoints[i] = .5
                segYPoints[i] = .5

        segCount = 100
        segVeloX = 0
        segVeloY = 0
        if random.randint(0, 1) == 0:
            segVeloX = (random.randint(0, 2)-1)/1000
        else:
            segVeloY = (random.randint(0, 2)-1)/1000
        segStart = random.randint(0, maxPoints-1)
        segEnd   = random.randint(segStart, maxPoints-1)
    points = []

    for i in range(segStart, segEnd):
        segXPoints[i] += (segVeloX*step)/2
        segYPoints[i] += (segVeloY*step)/2

        if (segXPoints[i]>1 or segXPoints[i]<0):
            segXPoints[i] = .5
        if (segYPoints[i]>1 or segYPoints[i]<0 ):
            segYPoints[i] = .5

    for i in range(maxPoints):
        points.append((segXPoints[i],segYPoints[i]))

    return points, "RectShift"


#=============================================================================== 40
#segmentshift

segXPoints = [0.5] * 1000
segYPoints = [0.5] * 1000
segStart = 0
segEnd   = 0
segVeloX=0
segVeloY=0
segCount = 1

def generateL40(seed, maxPoints):


    global pSegSeed,segXPoints,segYPoints,segStart,segEnd,segVeloX,segVeloY,segCount,segCount
    random.seed(seed)
    step = seed-pSegSeed
    pSegSeed = seed

    segCount -= 1
    if (segCount <= 0):
        if random.randint(0, 1) == 0:
            # reset some points to the position of the first in this sequence
            startI = random.randint(0, maxPoints-1)
            endI = random.randint(startI, maxPoints-1)
            for i in range(startI,endI):
                segXPoints[i] = segXPoints[startI]
                segYPoints[i] = segYPoints[startI]

        segCount = 100
        if random.randint(0, 1) == 0:
            segVeloX = (random.randint(0, 2)-1)/1000
        else:
            segVeloY = (random.randint(0, 2)-1)/1000
        segStart = random.randint(0, maxPoints-1)
        segEnd   = random.randint(segStart, maxPoints-1)
    points = []

    for i in range(segStart, segEnd):
        segXPoints[i] += (segVeloX*step)/5
        segYPoints[i] += (segVeloY*step)/5

        if (segXPoints[i]>1 or segXPoints[i]<0):
            segXPoints[i] = .5
        if (segYPoints[i]>1 or segYPoints[i]<0 ):
            segYPoints[i] = .5

    for i in range(maxPoints):
        points.append((segXPoints[i],segYPoints[i]))

    return points, "SegmentShift"


#=============================================================================== 39
#rcircleworm

wormAngles   = [0,0,0,0,0,0,0,0,0,0,0]
targetPoints = []
smoothPoints = []


def generateL39(seed, maxPoints):

    random.seed(seed)
    global wormAngles, targetPoints

    if(random.random()<.01):
        wormAngles[round(random.random()*10-.5)]=(random.random()-.5)*.5

    newAngle = wormAngles[0]
    angle = 0
    cornerCount = 0
    x = .5
    y = .5
    filter = 50
    points = []
    for i in range(maxPoints):
        angle += wormAngles[round(i/maxPoints*10)]
        x=x+math.sin(angle)/1000
        y=y-math.cos(angle)/1000
        if i < len(targetPoints): # If it exists, update the point
            targetPoints[i] = (x, y)
            smoothPoints[i] = ((smoothPoints[i][0]*filter+x)/(filter+1),(smoothPoints[i][1]*filter+y)/(filter+1))

        else:
            targetPoints.append((x, y))
            smoothPoints.append((.5,.5))
        points.append((smoothPoints[i][0],smoothPoints[i][1],1))


    return points, "Circleworm"




#=============================================================================== 38
#random zappel
zappelPoints = []
zappelInit = 1

def generateL38(seed, maxPoints):

    random.seed(seed)
    global zappelPoints, zappelInit

    if zappelInit == 1:
        zappelInit = 0
        for i in range(maxPoints):
            x = random.random()
            y = random.random()
            if i > 0:
                x = zappelPoints[i-1][0] + (random.random() - 0.5) / 100
                y = zappelPoints[i-1][1] + (random.random() - 0.5) / 100
            zappelPoints.append((x, y))  # Append initial points
    else:
        for i in range(maxPoints):
            x = random.random()
            y = random.random()
            getNewRnd = math.sin((i+seed)/100)+1
            if (i > 0):
                # Create new x, y values based on the previous point
                new_x = min(1,max(0,(zappelPoints[i-1][0] + zappelPoints[i][0] + (random.random() - 0.5)*getNewRnd/500)/2))
                new_y = min(1,max(0,(zappelPoints[i-1][1] + zappelPoints[i][1] + (random.random() - 0.5)*getNewRnd/500)/2))
                # Replace the tuple at index i with a new one
                zappelPoints[i] = (new_x, new_y)
            else:
                # Create new x, y values based on the previous point
                new_x = min(1,max(0,(zappelPoints[maxPoints-1][0] + zappelPoints[i][0] + (random.random() - 0.5)*getNewRnd/500)/2))
                new_y = min(1,max(0,(zappelPoints[maxPoints-1][1] + zappelPoints[i][1] + (random.random() - 0.5)*getNewRnd/500)/2))
                # Replace the tuple at index i with a new one
                zappelPoints[i] = (new_x, new_y)

    # Generate the points list with (x, y, 1)
    points = [(zappelPoints[i][0], zappelPoints[i][1], 1) for i in range(maxPoints)]
    return points, "R_Zappel"


#=============================================================================== 37

# 4x4 grid
def generateL37(seed, maxPoints):

    sec = seed / 100
    points = []

    for i in range(maxPoints):
        x = (round((i+seed+(math.sin(seed/432.1)+1)*300)/(123))%5)/4
        y = (round((i+seed*.768+(math.sin(seed/123.4)+1)*300)/(321))%5)/4

        points.append((x*.6+.2,y*.6+.2,1))
    return points, "4x4 Grid"



#=============================================================================== 36

# rotating line
secRound = 0
lastSecRound = 0
targetAngle = [0,0,0,0]
angle = [0,0,0,0]
def generateL36(seed, maxPoints):

    random.seed(seed)
    global lastSecRound, secRound, angle, targetAngle
    secRound = round(seed / 100)

    for i in range(4):
        if(secRound!=lastSecRound and secRound%4 == i):
            targetAngle[i]=angle[i]+(random.random()-.5)*3.14

        points = []
        angle[i] = (angle[i]*19+targetAngle[i])/20

    lastSecRound = secRound

    shift = (math.sin(seed*.001)+1)/2
    for i in range(maxPoints):
        i4 = min(3,max(0,round(i/maxPoints*(3.5+shift)-.5)))
        x1 = ((i%round(maxPoints/4))/(maxPoints/4))-.5
        y1 = 0

        x2 = x1 * math.cos(angle[i4]) + y1 * -math.sin(angle[i4]);
        y2 = x1 * math.sin(angle[i4]) + y1 * math.cos(angle[i4]);


        points.append((x2+.5,y2+.5,1))
    return points, "Rotating Line"


#=============================================================================== 35

# checkered
def generateL35(seed, maxPoints):

    sec = seed / 100
    points = []

    for i in range(maxPoints):
        x = 0
        y = 0
        s = (seed/100+i/2)
        m1 = stretch(math.sin(seed/1000),-1,1,0,.3)
        m2 = stretch(math.sin(seed/1234),-1,1,0,.3)
        if (i<maxPoints/2):
            x = round((i+seed)/100+((math.sin(s*.013)+math.sin(s*.41)))*.2)%2 *(1-m1*2) +m1
            y = i/(maxPoints/2)
        else:
            x = (i-(maxPoints/2))/(maxPoints/2)
            y = round((i+seed)/101+((math.sin(s*.39) +math.sin(s*.011)))*.3) %2 *(1-m2*2) +m2

        points.append((x,y,1))
    return points, "Checkered"


#=============================================================================== 34
# agents
ax = [0] * 20 # position
ay = [0] * 20
adx = [0] * 20 # direction
ady = [0] * 20
adi = [0] * 20 # distance



def generateL34(seed, maxPoints):

    random.seed(seed)
    sec = seed / 100
    points = []
    # move agents
    for i in range(len(ax)):
        if (adi[i]<=0):
            # get new distance & direction
            adi[i] = round(random.random()*30)
            if adx[i] !=0:

                adx[i] = 0
                ady[i] = .1
                if (random.random()>.5):
                    ady[i] = -.1
            else:
                ady[i] = 0
                adx[i] = .1
                if (random.random()>.5):
                    adx[i] = -.1
        ax[i] += adx[i] # move agent in x
        ay[i] += ady[i] # move agent in y
        ax[i] *= .999
        ay[i] *= .999
        adi[i] -= .1     # reduce distance

    # Fill the array

    for i in range(maxPoints):
        p = max(min(round(i/maxPoints*8),18),0)
        o = max(min((i/maxPoints*8+.5),18),0)-p
        if (o<.5):
            tx = lerp(o*2, ax[p],ax[p+1])
            ty = ay[p]
        else:
            tx = ax[p+1]
            ty = lerp((o-.5)*2, ay[p],ay[p+1])

        # Perform rotation
        angle = 45
        x = tx * math.cos(angle) - ty * math.sin(angle)
        y = tx * math.sin(angle) + ty * math.cos(angle)
        points.append((x/100+.5,y/100+.5,1))
    return points, "Agents"

#=============================================================================== 33
# zzzzz grid
def generateL33(seed, maxPoints):

    sec = seed / 100
    points = []
    # Fill the array
    for i in range(maxPoints):
        p = soft_step((i/maxPoints-.5)*sin_osc(sec*.0014327,500,2000)/100)*100
        xShift = round((p%300)/100-.5)/3
        yShift = (round(p/300-.5)/3)%1
        o = (p%100)/100-.5
        freq = (math.sin((xShift*3+yShift+math.sin(sec*.0002)*.2)*123)+15)*7
        scale = .1
        x1 = (saw_osc(p/freq)-.5)*scale
        y1 = o*scale
        angle = math.sin(round(p/100-.5)+sec*.03)*3+sec*.003
        # Perform rotation
        x = x1 * math.cos(angle) - y1 * math.sin(angle)
        y = x1 * math.sin(angle) + y1 * math.cos(angle)
        points.append((y+xShift+.1666,x+yShift+.1666,1))
    return points, "ZZ Grid"

#=============================================================================== 32
# polygon
def generateL32(seed, maxPoints):

    sec = seed / 20
    points = []
    # Fill the array
    a=(soft_step(sin_osc(sec*.0146+math.sin(sec*.00556),5,11))+1)/2+.001
    a1=(soft_step(sin_osc(seed/21123,0,10))*5)/5
    a2=(soft_step(sin_osc(seed/21367,0,10))*5)/5
    for i in range(maxPoints):

        p =(round((i/maxPoints-.5)*a-.5)/a * math.pi*5)
        x=math.sin(p+sec*.0125+math.sin(sec*.03556))*.4
        y=math.cos(p+sec*.0125+math.sin(sec*.03556))*.4

        x,y=project_to_sphere(x,y,a1,a2)


        points.append((x/2+.5,y/2+.5,1))
    return points, "Polygon"

#=============================================================================== 31
# zzzzz2
shiftIndex=0
def generateL31(seed, maxPoints):

    global shiftIndex
    sec = seed / 100
    points = []
    pointsShifted = []
    # Fill the array
    for i in range(maxPoints):
        p = i/maxPoints*1100
        xShift = round((p%300)/100-.5)/3
        yShift = (round(p/300-.5)/3)%1
        o = (p%100)/100-.5
        freq = (math.sin((xShift*3+yShift+math.sin(sec*.0002)*.2)*123)+2)*7
        scale = (freq/200)+.05
        x1 = (saw_osc(p/freq)-.5)*scale
        y1 = o*scale
        angle = math.sin(round(p/100-.5)+sec*.03)*3+sec*.003
        # Perform rotation
        x = x1 * math.cos(angle) - y1 * math.sin(angle)
        y = x1 * math.sin(angle) + y1 * math.cos(angle)
        points.append((x+xShift+.1666,y+yShift+.1666,1))
    #shift beginning and end
    shiftIndex += 1
    for i in range(maxPoints):
        x,y,d = points[(i+shiftIndex)%maxPoints]
        pointsShifted.append((x,y,1))
    return pointsShifted, "Z2"

#=============================================================================== 30
# zzzzz boxes
def generateL30(seed, maxPoints):

    sec = seed / 100
    points = []
    # Fill the array
    for i in range(maxPoints):
        p = i/maxPoints*200
        p += sin_osc(sec/314,0,50)
        yShift = round(p/sin_osc(sec/123,100,500))/4
        xShift = round((p%200)/sin_osc(sec/114,150,250))/4
        w = p%200
        x=saw_osc(w/sin_osc(sec/245+(xShift*10+yShift*20),3,15))/5
        y=w/500
        if(round((p%200)/100)%2==0):
            e=x
            x=y
            y=e

        points.append((x+xShift,y+yShift,1))
    return points, "ZZ Boxes"

#=============================================================================== 29
# 2d zigzag
def generateL29(seed, maxPoints):

    sec = seed / 100
    points = []
    c=sec/1000
    # Fill the array
    for i in range(maxPoints):
        p = i/maxPoints*1000
        lens=pow((math.sin(p*math.pi/1000)),10)
        x=  ((saw_osc(round((p+sec)*.0135)/8.4)*2)/2-.5) * ((saw_osc(round((p)*.043)/21.75)*2)/2-.5)
        x*=lens
        y=  ((saw_osc(round((p+sec)*.013)/18.5)*2)/2-.5) * ((saw_osc(round((p)*.041)/12.75)*2)/2-.5)
        y*=lens
        points.append((x*2+.5,y*2+.5,1))
    return points, "2D Zig Zag"


#=============================================================================== 28
# hirozontal zigzag
def generateL28(seed, maxPoints):

    sec = seed / 100
    points = []
    c=sec/10
    # Fill the array
    for i in range(maxPoints):
        p = i/maxPoints*1000
        x=((p+sec)%1000)/1000
        y=  ((saw_osc(soft_step((p+sec+c)*.3)/12.5+sec/300)*2)/2-.5) * ((saw_osc(soft_step((p+sec+c)*.041)/1.275+sec/200)*2)/2-.5) * ((saw_osc(soft_step((p+sec+c)*.0061)/12.95+sec/100)*2)/2-.5)
        y*=(math.sin(x*math.pi))
        points.append((x*.8+.1,y+.5,1))
    return points, "Horizontal Zig Zag"

#=============================================================================== 27
# shiftCircles
def generateL27(seed, maxPoints):

    sec = seed / 1000
    points = []
    # Fill the array
    for i in range(maxPoints):
        p = i/maxPoints*10
        radius=.05
        x=(round(p+sec/20)%10)/10
        y=0
        if round(p)%2==0:
            x+=math.sin((sec+p*10.7))*radius
            y+=math.cos((sec+p*10.7))*radius
        points.append((y*.8+.5,x*.8+.125,1))
    return points, "Shift Circles"

#=============================================================================== 26
# cubes
def generateL26(seed, maxPoints):

    sec = seed / 1000
    points = []
    # Example coordinates
    coords = [-1,-1,-1],[1,-1,-1],[1,1,-1],[-1,1,-1],[-1,-1,-1],[-1,-1,1],[1,-1,1],[1,1,1],[-1,1,1],[-1,-1,1],[1,-1,1],[1,-1,-1],[1,1,-1],[1,1,1],[-1,1,1],[-1,1,-1]
    # Fill the array
    for i in range(maxPoints):
        p = round(i / 10)
        p = p % len(coords)
        # Rotation angles (in radians)
        s = math.sin(sec*.01234)
        theta_x = sec / 1.47+ round(i/250)*s
        theta_y = sec / 1.66+ round(i/250)*s
        theta_z = sec / 1.31+ round(i/250)*s
        # Rotate coordinates
        rotated_coords = rotate_z(rotate_y(rotate_x(coords[p], theta_x), theta_y), theta_z)
        # Project onto 2D
        x,y = project_2d(rotated_coords)
        x = x*.05 + round(i/250)*.2+.1
        y=y*.05
        points.append((x,y+.5,1))
    return points, "Cubes"

#=============================================================================== 25
# 3d
def generateL25(seed, maxPoints):

    sec = seed / 4000
    points = []
    # Example coordinates
    coords = []
    for i in range(maxPoints):
        a = round(math.sin((sec+i)*.851)*1.1)
        b = round(math.sin((sec+i)*.453)*1.1)
        c = round(math.sin((sec+i)*.252)*1.1)
        coords.append((a,b,c))
    # Fill the array
    for i in range(maxPoints):
        p = round(i/35)
        p = p%len(coords)
        # Rotation angles (in radians)
        theta_x = sec / 1.147
        theta_y = sec / 1.166
        theta_z = sec / 1.131  #
        # Rotate coordinates
        rotated_coords = rotate_z(rotate_y(rotate_x(coords[p], theta_x), theta_y), theta_z)
        # Project onto 2D
        x,y = project_2d(rotated_coords)
        points.append((x*.2+.5,y*.2+.5,1))
    return points, "3D"

#=============================================================================== 24
# cube
def generateL24(seed, maxPoints):

    sec = seed / 1000
    points = []
    # Example coordinates
    coords = [-1,-1,-1],[1,-1,-1],[1,1,-1],[-1,1,-1],[-1,-1,-1],[-1,-1,1],[1,-1,1],[1,1,1],[-1,1,1],[-1,-1,1],[1,-1,1],[1,-1,-1],[1,1,-1],[1,1,1],[-1,1,1],[-1,1,-1]
    # Fill the array
    for i in range(maxPoints):
        p = round(i/35)
        p = p%len(coords)
        # Rotation angles (in radians)
        theta_x = sec / 1.47
        theta_y = sec / 1.66
        theta_z = sec / 1.31  #
        # Rotate coordinates
        rotated_coords = rotate_z(rotate_y(rotate_x(coords[p], theta_x), theta_y), theta_z)
        # Project onto 2D
        x,y = project_2d(rotated_coords)

        points.append((x*.2+.5,y*.2+.5,1))
    return points, "Cube"

#=============================================================================== 23
# 45° wobblegrid
def generateL23(seed, maxPoints):

    sec = seed / 1000
    points = []

    # Fill the array
    for i in range(maxPoints):
        x = sin_osc(sec/112+soft_step(sin_osc(sec/15.6+i/200+sin_osc((sec+i/5)*.0263,0,2),0,10)),0.1,.9)
        y = sin_osc(sec/113+soft_step(sin_osc(sec/31.9+i/200+sin_osc((sec+i/5)*.0223,0,2),0,10)),0.1,.9)
        points.append(rotate_coordinates(x, y, 45))
    return points, "45° Wobblegrid"

#=============================================================================== 22
# 45° grid
def generateL22(seed, maxPoints):

    sec=seed/1000
    points=[];

    # Fill the array
    for i in range(maxPoints):
        x = sin_osc(sec/110+round(sin_osc(sec/12.6+i/200+sin_osc((sec+i/5)*.023,0,2),0,10)),0.1,.9)
        y = sin_osc(sec/111+round(sin_osc(sec/42.9+i/200+sin_osc((sec+i/5)*.0123,0,2),0,10)),0.1,.9)
        points.append(rotate_coordinates(x, y, 45))
    return points, "45Deg Grid"

#=============================================================================== 21
#flexigrid wobble
def generateL21(seed, maxPoints):

    sec=seed/1000
    points=[];

    # Fill the array
    for i in range(maxPoints):
        x = sin_osc(sec/110+soft_step(sin_osc(sec/12.6+i/200+sin_osc((sec+i/5)*.023,0,2),0,10)),0.1,.9)
        y = sin_osc(sec/111+soft_step(sin_osc(sec/42.9+i/200+sin_osc((sec+i/5)*.0123,0,2),0,10)),0.1,.9)
        points.append((x,y,1))
    return points, "Flexigrid Wobble"

#=============================================================================== 20
#shaded circle
def generateL20(seed, maxPoints):
    seed = seed/10
    timeWobble = soft_step(sin_osc(seed/1000,0,10))*50
    timeWobble1 = soft_step(sin_osc(seed/19357,0,10))*50
    timeWobble2 = soft_step(sin_osc(seed/23753,0,10))*50
    timeWobble3 = soft_step(sin_osc(seed/47753,0,10))*50
    sec=seed/10+timeWobble
    points=[];
        # Fill the array

    flip = 0
    for i in range(maxPoints):
        p = i/maxPoints
        x=0
        y=0
        d1 = sin_osc(sec/1236+timeWobble1,.1,.4)
        d2 = sin_osc(sec/1647+timeWobble2,.1,.4)
        d3 = sin_osc(sec/1758+timeWobble3,.1,.4)
        if (p<.95): # outline circle
            x = math.sin(p/.91*2*math.pi+sec/110)*d3
            y = math.cos(p/.93*2*math.pi+sec/110)*d3
        else:
            flip = (flip+1)%4
            if flip <2:
                x = math.sin(p/.18*2*math.pi-sec/1110)*d1
                y = math.cos(p/.19*2*math.pi-sec/1110)*d1
            else:
                p=(i+1)/maxPoints
                x = -math.sin(p/.17*2*math.pi-sec/1110)*d2
                y = math.cos(p/.16*2*math.pi-sec/1110)*d2

        points.append((x+.5,y+.5,1))
    return points, "Shaded Circle"

#=============================================================================== 19b
def generateL19(seed, maxPoints):

    sec=seed/10
    points=[];

        # Fill the array

    for i in range(maxPoints):
        sec2=sec/4
        x=math.sin((math.sin((sec2+i)/13)*math.sin((sec2+i)/12.1))+sec2/1922)
        y=math.sin((math.cos((sec2+i)/15.1)*math.cos((sec2+i)/16.1))+sec2/1432)
        points.append((x*.5+.5,y*.5+.5,1))
    return points, "Cornermob"


#=============================================================================== 18
#cross
def generateL18(seed, maxPoints):

    random.seed(seed)
    global cooXb,cooYb,cooDirXb,cooDirYb,cooDistb
    sec=seed/1000
    points=[];

    for i in range(100):
        if (cooDistb[i]<=0):
            cooDistb[i] = random.random()* 200
            cooDirXb[i] = random.random()-.5
            cooDirYb[i] = random.random()-.5
        cooXb[i]+=cooDirXb[i]/100
        cooYb[i]+=cooDirYb[i]/100
        cooDistb[i]-=.5
    # Fill the array
    for i in range(maxPoints):
        ii = round(i/maxPoints*20-.5)
        x=cooXb[ii]
        y=cooYb[ii]
        points.append((soft_step(sin_osc(x,0,2))/2,soft_step(sin_osc(y,0,2))/2, 1))
    return points, "Cross"

#=============================================================================== 17
#zigzag
def generateL17(seed, maxPoints):

    random.seed(seed)
    global cooX,cooY,cooDirX,cooDirY,cooDist
    sec=seed/1000
    points=[];

    for i in range(10):
        if (cooDist[i]<=0):
            cooDist[i] = random.random()* 200
            cooDirX[i] = random.random()-.5
            cooDirY[i] = random.random()-.5
        cooX[i]=cooX[i]*.8+cooDirX[i]/10
        cooY[i]=cooY[i]*.8+cooDirY[i]/10
        cooDist[i]-=1


    # Fill the array

    for i in range(maxPoints):
        ii = round(i/maxPoints*10-.5)
        x=cooX[ii]
        y=cooY[ii]
        points.append((y*1.5+.5,x*1.5+.5, 1))
    return points, "Zig Zag"



#=============================================================================== 16
#portal pac man G
def generateL16(seed, maxPoints):


    sec=seed/1000
    points=[];
    # Fill the array

    for i in range(maxPoints):
        size = soft_step(sin_osc(sec*.0175,5,40))/80

        triggerAngle1 = soft_step(sin_osc(sec*.0165,.1,21))/22*maxPoints
        size1 = (sin_osc(sec*.0215,1,40))/80
        if(i>=triggerAngle1):
            size = size1

        triggerAngle2 = soft_step(sin_osc(sec*.0145,.1,21))/22*maxPoints
        size2 = (sin_osc(sec*.0255,-40,40))/80
        if(i>=triggerAngle2):
            size = size*.5 + size2

        triggerAngle3 = soft_step(sin_osc(sec*.0135,.1,21))/22*maxPoints
        size3 = (sin_osc(sec*.0235,-40,40))/80
        if(i>=triggerAngle3):
            size = size*.5 + size3

        triggerAngle4 = soft_step(sin_osc(sec*.0115,.1,21))/22*maxPoints
        size4 = (sin_osc(sec*.0275,-40,40))/80
        if(i>=triggerAngle4):
            size = size*.5 + size4

        angle = i/maxPoints * math.pi * 6
        x=math.sin(angle)*size/3+.5
        y=math.cos(angle)*size/3+.5
        points.append((y,x, 1))
    return points, "Portal"

#=============================================================================== 15
#grid
def generateL15(seed, maxPoints):

    sec=seed/1000
    sec*=10
    points=[];
    # Fill the array
    s1=saw_osc(sec/110.47)*saw_osc(sec/1010.27)*2+1
    s2=saw_osc(sec/310.57)*saw_osc(sec/990.19)*2+1
    for i in range(maxPoints):
        s3=4
        s4=4
        x=round(saw_osc(i/maxPoints*s1)*s3)/s3
        y=round(saw_osc(i/maxPoints*s2)*s4)/s4
        points.append((y,x, 1))
    return points, "Grid"

#=============================================================================== 14
#ud wobble line
def generateL14(seed, maxPoints):

    sec=seed/1000
    points=[];
    # Fill the array
    for i in range(maxPoints):
        x = (math.sin(sec*.761+i/113)*3)/6
        x *= (math.sin(sec*.753+i/134)*3)/6
        x *= (math.sin(sec*.77-i/176)*3)/6
        x *= (math.sin(sec*.7897-i/213)*3)/6
        x *= (sin_osc(sec*.0141+i*.1314,-5,5))
        y = i/maxPoints
        points.append((y+.5,x, 1))
    return points, "UD Wobble Line"

#=============================================================================== 13
#lr wobble line
def generateL13(seed, maxPoints):

    sec=seed/1000
    points=[];
    # Fill the array

    for i in range(maxPoints):
        x = (math.sin(sec*.761+i/113)*3)/6
        x *= (math.sin(sec*.753+i/134)*3)/6
        x *= (math.sin(sec*.77-i/176)*3)/6
        x *= (math.sin(sec*.7897-i/213)*3)/6
        x *= (sin_osc(sec*.0141+i*.1314,-5,5))
        y = i/maxPoints
        points.append((x+.5,y, 1))
    return points, "LR Wobble Line"

#=============================================================================== 12
#soft raster
def generateL12(seed, maxPoints):

    sec=seed/1000
    points=[];
    # Fill the array
    for i in range(maxPoints):
        f=sin_osc(sec*.013+i/100,1,6)
        g=sin_osc(sec*.043+i/100,1,6)
        x=soft_step(sin_osc(sin_osc(sec*.061+i*.00075,0,5),-f,f))/f+i/4000
        y=soft_step(sin_osc(sin_osc(sec*.072+i*.00075,0,5),-g,g))/g+i/4000
        points.append((x*.4+.5,y*.4+.5, 1))
    return points, "Soft Raster"

#=============================================================================== 11
# spinning disc
def generateL11(seed, maxPoints):

    sec=seed/500
    sec=soft_step(sec)/2
    points=[];
    # Fill the array
    for i in range(maxPoints):
        size = i/maxPoints*.3+.1

        rotation = sec*.0361+sin_osc(sec*.013,-1,1)+soft_step(sin_osc(sec*.0164,-i*30,i*30)/1000)
        angleWidth = soft_step(sin_osc(sec*.13432,1,10))/10


        angle = (soft_step(math.sin((i)/soft_step(sin_osc(sec*.018649,15,30))))*angleWidth+rotation)
        x = math.sin(angle)*size
        y = math.cos(angle)*size
        points.append((x+.5, y+.5, 1 ))
    return points, "Spinning Disc"

#=============================================================================== 10
# snake
def generateL10(seed, maxPoints):

    random.seed(seed)
    global snakeXPoints, snakeYPoints, snakeX, snakeY, snakeDX, snakeDY, snakeDist, snakeIndex

    if(snakeDist<=0):
        snakeDist=random.random()*50+1

        # new direction
        if (snakeDX!=0):
            snakeDX=0
            if (random.random()>=.5):
                snakeDY=1
            else:
                snakeDY=-1
            if snakeY>50:
                snakeDY=-1
            if snakeY<-50:
                snakeDY=1
        else:
            snakeDY=0
            if (random.random()>=.5):
                snakeDX=1
            else:
                snakeDX=-1
            if snakeX>50:
                snakeDX=-1
            if snakeX<-50:
                snakeDX=1


    snakeX += snakeDX * 1
    snakeY += snakeDY * 1
    snakeDist -= (1)
    snakeXPoints[snakeIndex] = snakeX
    snakeYPoints[snakeIndex] = snakeY
    snakeIndex += 1
    snakeIndex = snakeIndex % maxPoints


    points=[];
    # Fill the array
    snpCount=snakeIndex
    for i in range(maxPoints):

        points.append((snakeXPoints[snpCount]/200+.5, snakeYPoints[snpCount]/200+.5, 1 ))
        snpCount=(snpCount+1)%maxPoints
    return points, "Snake"


#=============================================================================== 9
# rectiflower
def generateL09(seed, maxPoints):

    seed/=4
    sec=seed/1000
    points=[];
    # Fill the array
    t = 500
    for i in range(maxPoints):
        p = i*(saw_osc(sec/27)*5+15)+seed
        s = sin_osc(p/1711,.1,.8)
        size = sin_osc(p/10,.25,.4)*s
        if(round(i/sin_osc((seed/178)*.32,50,250))%3==0 or round(i/sin_osc((seed/265)*.32,20,50))%5==0):
            size=(round(p/176-sec)%10+round(p/167+sec/2)%10+round(p/158-sec*3)%10)/80+.1
        x = math.cos(p/t)*size+.5
        y = math.sin(p/t)*size+.5
        points.append((x, y, 1 ))
    return points, "Rectiflower"

#=============================================================================== 8
#cheese
def generateL08(seed, maxPoints):

    sec=seed/1000
    points=[];
    # Fill the array
    for i in range(maxPoints):
        x=(round(i/sin_osc(sec/676,10,50))%10)/5+.1
        x-=round(x-.5)
        y=(round(i/sin_osc(sec/767,10,150))%10)/5+.1
        y-=round(y-.5)
        cSize= sin_osc((sec+i)/100,.01,.08)
        cy=math.sin(i/5+sec)*cSize
        cx=math.cos(i/5+sec)*cSize
        points.append((x+cx, y+cy, 1 ))
    return points, "Cheese"

#=============================================================================== 7
# flower
def generateL07(seed, maxPoints):

    seed/=10
    sec=seed/1000
    points=[];
    # Fill the array
    for i in range(maxPoints):
        size = soft_step(sin_osc((sec*.5746+i*math.sin(sec*.0156))/16,1,4))/10
        resX = sin_osc(sec/17.36,50,200)
        resY = sin_osc(sec/7.36,50,200)
        o = sin_osc(seed/999.9,7,13)
        a= sin_osc(seed/379.9,-.5,.5)
        x = int(sin_osc(soft_step(sec*.91+i/resX),.5-size,.5+size)*o+a)/o
        y = sin_osc(soft_step(sec*1.01+i/resY*.97),.5-size,.5+size)
        points.append((x, y, 1 ))
    return points, "Flower"


#=============================================================================== 6

# Jiggerlines
def generateL06(seed, maxPoints):

    random.seed(seed)
    sec=seed/1000
    points=[];
    # Fill the array
    for i in range(maxPoints):
        it = i/maxPoints*sin_osc(sec/13+i/976,5,50)
        random.seed(round(sec/10+it))
        xc  = stretch(random.random(),0,1,.2,.8)
        yc  = stretch(random.random(),0,1,.2,.8)
        res = sin_osc(sin_osc((sec+i/20)/120,10,100),10,100);
        d = ((i/maxPoints)-.5)/5
        x = round(math.sin(i/300+sec/5)*.2*res)/res + d
        y = round(math.cos(i/300+sec/5)*.2*res)/res + d
        points.append((xc+x, yc+y, 1 ))
    return points, "Jiggerlines"

#=============================================================================== 5
# Three spirals
def generateL05(seed, maxPoints):

    random.seed(seed)
    sec=seed/1000
    points=[];
    # Fill the array
    for i in range(maxPoints):
        it = i/maxPoints*3#sin_osc(sec/10+i/1000,15,50)
        random.seed(round(sec/10+it))
        xc  = stretch(random.random(),0,1,.15,.85)
        yc  = stretch(random.random(),0,1,.15,.85)
        mag = .2*(i/maxPoints)
        x = math.sin(i/30+sec/5)*mag
        y = math.cos(i/30+sec/5)*mag
        points.append((xc+x, yc+y, 1 ))
    return points, "Three Spirals"

#=============================================================================== 4
# Rectiwobble
def generateL04(seed, maxPoints):

    sec=seed/1000
    points=[];
    # Fill the array
    for i in range(maxPoints):
        xc = stretch(math.sin(soft_step(sec/7+i/(sin_osc(sec*.0018,30,70)))),-1,1,.1,.9)
        yc = stretch(math.cos(soft_step(sec/11+i/(sin_osc(sec*.00371,30,70)))),-1,1,.1,.9)
        points.append((xc, yc, 1 ))
    return points, "Rectiwobble"

#=============================================================================== 3
# Swings
def generateL03(seed, maxPoints):

    sec=seed/1000
    points=[];
    # Fill the array

    for i in range(maxPoints):
        xc = sin_osc(soft_step(sec*.278+i/50),.2,.8)
        yc = sin_osc(soft_step(sec*.151+2+i/50),.2,.8)
        mag=sin_osc(soft_step(sec+i)/1271,.005,.25)
        points.append(( math.sin(i/10)*mag + xc , math.cos(i/10)*mag + yc, 1 ))
    return points, "Swings"

#=============================================================================== 2
# Spirals
def generateL02(seed, maxPoints):

    sec=seed/1000
    points=[];
    # Fill the array
    for i in range(maxPoints):
        #xCenter
        xc=round(i/300)/sin_osc(sec/3+i/100,2,5)+sin_osc(sec/7+i/77,.05,.15);
        mag = sin_osc(sec/21+i/123,-.3,.3)
        points.append((math.sin(i/10+sec)*mag+xc, math.cos(i/10+sec)*mag+.5, 1))
    return points, "Spirals"


#=============================================================================== 1
# Spiral
def generateL01(seed, maxPoints):

    sec=seed/1000
    points=[];
    # Fill the array
    for i in range(maxPoints):
        mag = sin_osc((sec*3+i/sin_osc(sec/777,2,20))/80,-.4,.4)
        div = sin_osc(sec/327,15,35);
        circ = i/div+sec/37+sin_osc(sec/10,1,100);
        # Each point is represented as a tuple (x, y, d) where x and y are coordinates and d is draw or not
        points.append((math.sin(circ)*mag+.5, math.cos(circ)*mag+.5, 1))
    return points, "Spiral"

# Architectural Lines
def generateL00(seed, maxPoints):

    sec=seed/1000
    points=[];
    i=0
    oldSeed = -1
    oldX=0
    oldY=0
    while i<maxPoints:
        it = i/maxPoints*sin_osc(sec/21,3,30)
        newSeed = round(sec+it)
        random.seed(newSeed)
        if(oldSeed != newSeed):
            x=random.random()*.6+.2
            y=oldY
        else:
            x=random.random()*.6+.2
            y=random.random()*.6+.2
            oldX = (oldX*10+x)/11
            oldY = (oldY*10+y)/11
        oldSeed = newSeed

        zoom = sin_osc(sec/357136+i/17,.9,1.2)
        noZoom = sin_osc(sec/126,0,1)
        zoom = zoom * noZoom + (1-noZoom)
        x=(x-.5)*zoom+.5
        y=(y-.5)*zoom+.5
        points.append((x,y,1))
        i+=1
    return points, "Architecture"

################################################################################ arithmetic functions
import numpy as np

def project_to_sphere(x1, y1, rx, ry):
    # Precompute trigonometric values for efficiency
    sin_rx, cos_rx = np.sin(rx), np.cos(rx)
    sin_ry, cos_ry = np.sin(ry), np.cos(ry)

    # Compute combined rotation matrix R = Rx @ Ry
    R = np.array([
        [cos_ry, 0, sin_ry],
        [sin_rx * sin_ry, cos_rx, -sin_rx * cos_ry],
        [-cos_rx * sin_ry, sin_rx, cos_rx * cos_ry]
    ])

    # Convert square coordinates to spherical coordinates
    longitude = x1 * np.pi  # Map x1 from [-1,1] to [-π,π]
    latitude = y1 * np.pi / 2  # Map y1 from [-1,1] to [-π/2,π/2]

    # Convert to 3D cartesian coordinates on unit sphere
    cos_lat = np.cos(latitude)
    x, y, z = cos_lat * np.sin(longitude), np.sin(latitude), cos_lat * np.cos(longitude)

    # Apply combined rotation
    rotated_point = R @ np.array([x, y, z])

    # Simple orthographic projection - just take x and y coordinates
    x2, y2 = rotated_point[0], rotated_point[1]

    return x2, y2

"""
def project_to_sphere(x1, y1, rx, ry):

    # Convert square coordinates to spherical coordinates
    longitude = x1 * np.pi  # Map x1 from [-1,1] to [-π,π]
    latitude = y1 * np.pi/2  # Map y1 from [-1,1] to [-π/2,π/2]

    # Convert to 3D cartesian coordinates on unit sphere
    x = np.cos(latitude) * np.sin(longitude)
    y = np.sin(latitude)
    z = np.cos(latitude) * np.cos(longitude)

    # Create rotation matrices
    Rx = np.array([
        [1, 0, 0],
        [0, np.cos(rx), -np.sin(rx)],
        [0, np.sin(rx), np.cos(rx)]
    ])

    Ry = np.array([
        [np.cos(ry), 0, np.sin(ry)],
        [0, 1, 0],
        [-np.sin(ry), 0, np.cos(ry)]
    ])


    # Apply rotations
    point = np.array([x, y, z])
    rotated_point = Rx @ Ry @ point

    # Simple orthographic projection - just take x and y coordinates
    x2 = rotated_point[0]  # Already in [-1, 1] since it's on unit sphere
    y2 = rotated_point[1]  # Already in [-1, 1] since it's on unit sphere

    return x2, y2
"""

def rotate_and_project(x1, y1, rot_x, rot_y, rot_z) :

    # Convert input point to 3D coordinates (z=0)
    point = np.array([x1, y1, 0])

    # Create rotation matrices
    Rx = np.array([
        [1, 0, 0],
        [0, np.cos(rot_x), -np.sin(rot_x)],
        [0, np.sin(rot_x), np.cos(rot_x)]
    ])

    Ry = np.array([
        [np.cos(rot_y), 0, np.sin(rot_y)],
        [0, 1, 0],
        [-np.sin(rot_y), 0, np.cos(rot_y)]
    ])

    Rz = np.array([
        [np.cos(rot_z), -np.sin(rot_z), 0],
        [np.sin(rot_z), np.cos(rot_z), 0],
        [0, 0, 1]
    ])

    # Apply rotations (order: Z -> Y -> X)
    rotated_point = Rx @ Ry @ Rz @ point

    # Project to 2D by taking x and y coordinates
    x2, y2 = rotated_point[0], rotated_point[1]

    return x2, y2

# returns a sinewave between out_lower and out_upper
def sin_osc(value, out_lower, out_upper):
    return stretch(math.sin(value),-1,1,out_lower,out_upper)

# sawtooth oscillator
def saw_osc(value):
    return math.fabs((value+.5-round(value))*2-1)

# stretch function (similar to map in processing)
def stretch(value, in_lower, in_upper, out_lower, out_upper):
    if (in_upper-in_lower == 0):
         return 0
    # Normalize the value to the range [0, 1] within the input boundaries
    normalized = (value - in_lower) / (in_upper - in_lower)
    # Map the normalized value to the range within the output boundaries
    mapped_value = out_lower + normalized * (out_upper - out_lower)
    return mapped_value

# soft step - works a bit like round(), but softer with steps at every second number
def soft_step(j):
    s = round(j)%2;
    r = round((j+1)/2)-.5;
    if (s==0):
        r = math.sin(math.pi*(j-round(j)))/2 + round(j/2)
    return r*2

# linear interpolation
def lerp(t, a, b):
    return (1 - t) * a + t * b

def rotate_coordinates(x, y, angle):
    # Convert angle from degrees to radians
    angle_rad = math.radians(angle)

    # Perform rotation
    new_x = x * math.cos(angle_rad) - y * math.sin(angle_rad)+.5
    new_y = x * math.sin(angle_rad) + y * math.cos(angle_rad)-.208

    return new_x, new_y, 1


snakeXPoints = [0] * 1000
snakeYPoints = [0] * 1000
snakeX = 0
snakeY = 0
snakeDX = 1
snakeDY = 0
snakeDist = 0
snakeIndex = 0

#initialize snake
for i in range(1000):
    generateL10(1,1000)


cooX = [0]*10
cooY = [0]*10
cooDirX = [0]*10
cooDirY = [0]*10
cooDist = [0]*10

cooXb = [0]*100
cooYb = [0]*100
cooDirXb = [0]*100
cooDirYb = [0]*100
cooDistb = [0]*100

def rotate_x(coords, theta):
    rotation_matrix = [
        [1, 0, 0],
        [0, math.cos(theta), -math.sin(theta)],
        [0, math.sin(theta), math.cos(theta)]
    ]
    return [sum(a * b for a, b in zip(row, coords)) for row in rotation_matrix]

def rotate_y(coords, theta):
    rotation_matrix = [
        [math.cos(theta), 0, math.sin(theta)],
        [0, 1, 0],
        [-math.sin(theta), 0, math.cos(theta)]
    ]
    return [sum(a * b for a, b in zip(row, coords)) for row in rotation_matrix]

def rotate_z(coords, theta):
    rotation_matrix = [
        [math.cos(theta), -math.sin(theta), 0],
        [math.sin(theta), math.cos(theta), 0],
        [0, 0, 1]
    ]
    return [sum(a * b for a, b in zip(row, coords)) for row in rotation_matrix]

def project_2d(coords):
    return (coords[0], coords[1])

def distance(x1, y1, x2, y2):
    # Euclidean distance formula
    distance = math.sqrt((x2 - x1)**2 + (y2 - y1)**2)
    return distance

def generatePoints(algorithm, seed, maxPoints):
    seed = seed // seedDivider
    if   (algorithm==0):
        return generateL00(seed, maxPoints)
    elif (algorithm == 1):
        return generateL01(seed, maxPoints)
    elif (algorithm == 2):
        return generateL02(seed, maxPoints)
    elif (algorithm == 3):
        return generateL03(seed, maxPoints)
    elif (algorithm == 4):
        return generateL04(seed, maxPoints)
    elif (algorithm == 5):
        return generateL05(seed, maxPoints)
    elif (algorithm == 6):
        return generateL06(seed, maxPoints)
    elif (algorithm == 7):
        return generateL07(seed, maxPoints)
    elif (algorithm == 8):
        return generateL08(seed, maxPoints)
    elif (algorithm == 9):
        return generateL09(seed, maxPoints)
    elif (algorithm == 10):
        return generateL10(seed, maxPoints)
    elif (algorithm == 11):
        return generateL11(seed, maxPoints)
    elif (algorithm == 12):
        return generateL12(seed, maxPoints)
    elif (algorithm == 13):
        return generateL13(seed, maxPoints)
    elif (algorithm == 14):
        return generateL14(seed, maxPoints)
    elif (algorithm == 15):
        return generateL15(seed, maxPoints)
    elif (algorithm == 16):
        return generateL16(seed, maxPoints)
    elif (algorithm == 17):
        return generateL17(seed, maxPoints)
    elif (algorithm == 18):
        return generateL18(seed, maxPoints)
    elif (algorithm == 19):
        return generateL19(seed, maxPoints)
    elif (algorithm == 20):
        return generateL20(seed, maxPoints)
    elif (algorithm == 21):
        return generateL21(seed, maxPoints)
    elif (algorithm == 22):
        return generateL22(seed, maxPoints)
    elif (algorithm == 23):
        return generateL23(seed, maxPoints)
    elif (algorithm == 24):
        return generateL24(seed, maxPoints)
    elif (algorithm == 25):
        return generateL25(seed, maxPoints)
    elif (algorithm == 26):
        return generateL26(seed, maxPoints)
    elif (algorithm == 27):
        return generateL27(seed, maxPoints)
    elif (algorithm == 28):
        return generateL28(seed, maxPoints)
    elif (algorithm == 29):
        return generateL29(seed, maxPoints)
    elif (algorithm == 30):
        return generateL30(seed, maxPoints)
    elif (algorithm == 31):
        return generateL31(seed, maxPoints)
    elif (algorithm == 32):
        return generateL32(seed, maxPoints)
    elif (algorithm == 33):
        return generateL33(seed, maxPoints)
    elif (algorithm == 34):
        return generateL34(seed, maxPoints)
    elif (algorithm == 35):
        return generateL35(seed, maxPoints)
    elif (algorithm == 36):
        return generateL36(seed, maxPoints)
    elif (algorithm == 37):
        return generateL37(seed, maxPoints)
    elif (algorithm == 38):
        return generateL38(seed, maxPoints)
    elif (algorithm == 39):
        return generateL39(seed, maxPoints)
    elif (algorithm == 40):
        return generateL40(seed, maxPoints)
    elif (algorithm == 41):
        return generateL41(seed, maxPoints)
    elif (algorithm == 42):
        return generateL42(seed, maxPoints)
    elif (algorithm == 43):
        return generateL43(seed, maxPoints)
    elif (algorithm == 44):
        return generateL44(seed, maxPoints)
    elif (algorithm == 45):
        return generateL45(seed, maxPoints)
    elif (algorithm == 46):
        return generateL46(seed, maxPoints)
    elif (algorithm == 47):
        return generateL47(seed, maxPoints)
    elif (algorithm == 48):
        return generateL48(seed, maxPoints)
    elif (algorithm == 49):
        return generateL49(seed, maxPoints)
    elif (algorithm == 50):
        return generateL50(seed, maxPoints)
    elif (algorithm == 51):
        return generateL51(seed, maxPoints)
    elif (algorithm == 52):
        return generateL52(seed, maxPoints)
    elif (algorithm == 53):
        return generateL53(seed, maxPoints)
    elif (algorithm == 54):
        return generateL54(seed, maxPoints)
    elif (algorithm == 55):
        return generateL55(seed, maxPoints)
    elif (algorithm == 56):
        return generateL56(seed, maxPoints)
    elif (algorithm == 57):
        return generateL57(seed, maxPoints)
    elif (algorithm == 58):
        return generateL58(seed, maxPoints)
    elif (algorithm == 59):
        return generateL59(seed, maxPoints)
    elif (algorithm == 60):
        return generateL60(seed, maxPoints)
    elif (algorithm == 61):
        return generateL61(seed, maxPoints)
    elif (algorithm == 62):
        return generateL62(seed, maxPoints)
    elif (algorithm == 63):
        return generateL63(seed, maxPoints)
    elif (algorithm == 64):
        return generateL64(seed, maxPoints)
    elif (algorithm == 65):
        return generateL65(seed, maxPoints)
    elif (algorithm == 66):
        return generateL66(seed, maxPoints)
    elif (algorithm == 67):
        return generateL67(seed, maxPoints)
    elif (algorithm == 68):
        return generateL68(seed, maxPoints)
    elif (algorithm == 69):
        return generateL69(seed, maxPoints)
    elif (algorithm == 70):
        return generateL70(seed, maxPoints)
    elif (algorithm == 71):
        return generateL71(seed, maxPoints)
    elif (algorithm == 72):
        return generateL72(seed, maxPoints)
    elif (algorithm == 73):
        return generateL73(seed, maxPoints)
    elif (algorithm == 74):
        return generateL74(seed, maxPoints)
    elif (algorithm == 75):
        return generateL75(seed, maxPoints)
    elif (algorithm == 76):
        return generateL76(seed, maxPoints)
    elif (algorithm == 77):
        return generateL77(seed, maxPoints)
    elif (algorithm == 78):
        return generateL78(seed, maxPoints)
    elif (algorithm == 79):
        return generateL79(seed, maxPoints)
numberOfAlgorithms = 79
